@@ -20,8 +20,10 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from stevedore import enabled
 
+from blazar import context
 from blazar.db import api as db_api
 from blazar.db import exceptions as db_ex
+from blazar import enforcement
 from blazar import exceptions as common_ex
 from blazar import manager
 from blazar.manager import exceptions
@@ -38,11 +40,13 @@ manager_opts = [
                      'support.)'),
     cfg.IntOpt('minutes_before_end_lease',
                default=60,
+               min=0,
                help='Minutes prior to the end of a lease in which actions '
                     'like notification and snapshot are taken. If this is '
                     'set to 0, then these actions are not taken.'),
     cfg.IntOpt('event_max_retries',
                default=1,
+               min=0,
                max=50,
                help='Number of times to retry an event action.')
 ]
@@ -69,10 +73,15 @@ class ManagerService(service_utils.RPCServer):
         self.plugins = self._get_plugins()
         self.resource_actions = self._setup_actions()
         self.monitors = monitor.load_monitors(self.plugins)
+        self.enforcement = enforcement.UsageEnforcement()
 
     def start(self):
         super(ManagerService, self).start()
-        self.tg.add_timer(EVENT_INTERVAL, self._event)
+        # NOTE(jakecoll): stop_on_exception=False was added because database
+        # exceptions would prevent threads from being scheduled again.
+        # TODO(jakecoll): Find a way to test this.
+        self.tg.add_timer_args(EVENT_INTERVAL, self._event,
+                               stop_on_exception=False)
         for m in self.monitors:
             m.start_monitoring()
 
@@ -153,6 +162,10 @@ class ManagerService(service_utils.RPCServer):
 
         LOG.info("Trying to execute events: %s", events)
         for event in events:
+            if not status.LeaseStatus.is_stable(event['lease_id']):
+                LOG.info("Skip event %s because the status of the lease %s "
+                         "is still transitional", event, event['lease_id'])
+                continue
             db_api.event_update(event['id'],
                                 {'status': status.event.IN_PROGRESS})
             try:
@@ -184,11 +197,14 @@ class ManagerService(service_utils.RPCServer):
             else:
                 db_api.event_update(event['id'],
                                     {'status': status.event.ERROR})
-                LOG.exception('Error occurred while handling event.')
+                LOG.exception('Error occurred while handling %s event for '
+                              'lease %s.', event['event_type'],
+                              event['lease_id'])
         except Exception:
             db_api.event_update(event['id'],
                                 {'status': status.event.ERROR})
-            LOG.exception('Error occurred while handling event.')
+            LOG.exception('Error occurred while handling %s event for '
+                          'lease %s.', event['event_type'], event['lease_id'])
         else:
             lease = db_api.lease_get(event['lease_id'])
             with trusts.create_ctx_from_trust(lease['trust_id']) as ctx:
@@ -203,6 +219,45 @@ class ManagerService(service_utils.RPCServer):
                                          date_format=date_format)
         return date
 
+    def _parse_lease_dates(self, start_date, end_date):
+        now = datetime.datetime.utcnow()
+        now = datetime.datetime(now.year,
+                                now.month,
+                                now.day,
+                                now.hour,
+                                now.minute)
+        if start_date == 'now':
+            start_date = now
+        else:
+            start_date = self._date_from_string(start_date)
+
+        if end_date == 'now':
+            end_date = now
+        else:
+            end_date = self._date_from_string(end_date)
+
+        return start_date, end_date, now
+
+    def _check_for_invalid_date_inputs(self, lease, values, now):
+        if (lease['start_date'] < now and
+                values['start_date'] != lease['start_date']):
+            raise common_ex.InvalidInput(
+                'Cannot modify the start date of already started leases')
+
+        if (lease['start_date'] > now and
+                values['start_date'] < now):
+            raise common_ex.InvalidInput(
+                'Start date must be later than current date')
+
+        if lease['end_date'] < now:
+            raise common_ex.InvalidInput(
+                'Terminated leases can only be renamed')
+
+        if (values['end_date'] < now or
+           values['end_date'] < values['start_date']):
+            raise common_ex.InvalidInput(
+                'End date must be later than current and start date')
+
     def validate_params(self, values, required_params):
         if isinstance(required_params, list):
             required_params = set(required_params)
@@ -213,7 +268,7 @@ class ManagerService(service_utils.RPCServer):
     def get_lease(self, lease_id):
         return db_api.lease_get(lease_id)
 
-    def list_leases(self, project_id=None):
+    def list_leases(self, project_id=None, query=None):
         return db_api.lease_list(project_id)
 
     def create_lease(self, lease_values):
@@ -237,20 +292,8 @@ class ManagerService(service_utils.RPCServer):
             self.validate_params(res, ['resource_type'])
 
         # Create the lease without the reservations
-        start_date = lease_values['start_date']
-        end_date = lease_values['end_date']
-
-        now = datetime.datetime.utcnow()
-        now = datetime.datetime(now.year,
-                                now.month,
-                                now.day,
-                                now.hour,
-                                now.minute)
-        if start_date == 'now':
-            start_date = now
-        else:
-            start_date = self._date_from_string(start_date)
-        end_date = self._date_from_string(end_date)
+        start_date, end_date, now = self._parse_lease_dates(
+            lease_values['start_date'], lease_values['end_date'])
 
         if start_date < now:
             raise common_ex.InvalidInput(
@@ -268,6 +311,15 @@ class ManagerService(service_utils.RPCServer):
             lease_values['start_date'] = start_date
             lease_values['end_date'] = end_date
 
+            allocations = self._allocation_candidates(
+                lease_values, reservations)
+            try:
+                self.enforcement.check_create(
+                    context.current(), lease_values, reservations, allocations)
+            except common_ex.NotAuthorized as e:
+                LOG.error("Enforcement checks failed. %s", str(e))
+                raise common_ex.NotAuthorized(e)
+
             events.append({'event_type': 'start_lease',
                            'time': start_date,
                            'status': status.event.UNDONE})
@@ -344,6 +396,17 @@ class ManagerService(service_utils.RPCServer):
                     self._send_notification(lease, ctx, events=['create'])
                     return lease
 
+    def _add_resource_type(self, reservations, existing_reservations):
+        rsvns_by_id = {}
+
+        for r in existing_reservations:
+            rsvns_by_id[r['id']] = r
+        for r in reservations:
+            if 'resource_type' not in r:
+                r['resource_type'] = rsvns_by_id[r['id']]['resource_type']
+
+        return reservations
+
     @status.lease.lease_status(
         transition=status.lease.UPDATING, result_in=status.lease.STABLE)
     def update_lease(self, lease_id, values):
@@ -363,42 +426,13 @@ class ManagerService(service_utils.RPCServer):
             datetime.datetime.strftime(lease['end_date'], LEASE_DATE_FORMAT))
         before_end_date = values.get('before_end_date', None)
 
-        now = datetime.datetime.utcnow()
-        now = datetime.datetime(now.year,
-                                now.month,
-                                now.day,
-                                now.hour,
-                                now.minute)
-        if start_date == 'now':
-            start_date = now
-        else:
-            start_date = self._date_from_string(start_date)
-        if end_date == 'now':
-            end_date = now
-        else:
-            end_date = self._date_from_string(end_date)
+        start_date, end_date, now = self._parse_lease_dates(start_date,
+                                                            end_date)
 
         values['start_date'] = start_date
         values['end_date'] = end_date
 
-        if (lease['start_date'] < now and
-                values['start_date'] != lease['start_date']):
-            raise common_ex.InvalidInput(
-                'Cannot modify the start date of already started leases')
-
-        if (lease['start_date'] > now and
-                values['start_date'] < now):
-            raise common_ex.InvalidInput(
-                'Start date must be later than current date')
-
-        if lease['end_date'] < now:
-            raise common_ex.InvalidInput(
-                'Terminated leases can only be renamed')
-
-        if (values['end_date'] < now or
-           values['end_date'] < values['start_date']):
-            raise common_ex.InvalidInput(
-                'End date must be later than current and start date')
+        self._check_for_invalid_date_inputs(lease, values, now)
 
         with trusts.create_ctx_from_trust(lease['trust_id']):
             if before_end_date:
@@ -410,22 +444,66 @@ class ManagerService(service_utils.RPCServer):
                     LOG.error("Invalid before_end_date param. %s", str(e))
                     raise e
 
-            # TODO(frossigneux) rollback if an exception is raised
             reservations = values.get('reservations', [])
-            for reservation in (
-                    db_api.reservation_get_all_by_lease_id(lease_id)):
+            existing_reservations = (
+                db_api.reservation_get_all_by_lease_id(lease_id))
+
+            try:
+                invalid_ids = set([r['id'] for r in reservations]).difference(
+                    [r['id'] for r in existing_reservations])
+            except KeyError:
+                raise exceptions.MissingParameter(param='reservation ID')
+
+            if invalid_ids:
+                raise common_ex.InvalidInput(
+                    'Please enter valid reservation IDs. Invalid reservation '
+                    'IDs are: %s' % ','.join([str(id) for id in invalid_ids]))
+
+            # Populate resource_type if missing from data passed by client
+            reservations = self._add_resource_type(reservations,
+                                                   existing_reservations)
+
+            try:
+                [
+                    self.plugins[r['resource_type']] for r
+                    in (reservations + existing_reservations)]
+            except KeyError:
+                raise exceptions.CantUpdateParameter(param='resource_type')
+
+            existing_allocs = self._existing_allocations(existing_reservations)
+
+            if reservations:
+                new_reservations = reservations
+                new_allocs = self._allocation_candidates(values,
+                                                         existing_reservations)
+            else:
+                # User is not updating reservation parameters, e.g., is only
+                # adjusting lease start/end dates.
+                new_reservations = existing_reservations
+                new_allocs = existing_allocs
+
+            try:
+                self.enforcement.check_update(context.current(), lease, values,
+                                              existing_allocs, new_allocs,
+                                              existing_reservations,
+                                              new_reservations)
+            except common_ex.NotAuthorized as e:
+                LOG.error("Enforcement checks failed. %s", str(e))
+                raise common_ex.NotAuthorized(e)
+
+            # TODO(frossigneux) rollback if an exception is raised
+            for reservation in existing_reservations:
                 v = {}
                 v['start_date'] = values['start_date']
                 v['end_date'] = values['end_date']
                 try:
                     v.update([r for r in reservations
                               if r['id'] == reservation['id']].pop())
-                except KeyError:
-                    raise exceptions.MissingParameter(param='reservation ID')
                 except IndexError:
                     pass
                 resource_type = v.get('resource_type',
                                       reservation['resource_type'])
+
                 if resource_type != reservation['resource_type']:
                     raise exceptions.CantUpdateParameter(
                         param='resource_type')
@@ -478,36 +556,58 @@ class ManagerService(service_utils.RPCServer):
                                result_in=(status.lease.ERROR,))
     def delete_lease(self, lease_id):
         lease = self.get_lease(lease_id)
-        if (datetime.datetime.utcnow() >= lease['start_date'] and
-                datetime.datetime.utcnow() <= lease['end_date']):
-            start_event = db_api.event_get_first_sorted_by_filters(
-                'lease_id',
-                'asc',
-                {
-                    'lease_id': lease_id,
-                    'event_type': 'start_lease'
-                }
-            )
-            if not start_event:
-                raise common_ex.BlazarException(
-                    'start_lease event for lease %s not found' % lease_id)
-            end_event = db_api.event_get_first_sorted_by_filters(
-                'lease_id',
-                'asc',
-                {
-                    'lease_id': lease_id,
-                    'event_type': 'end_lease',
-                    'status': status.event.UNDONE
-                }
-            )
-            if not end_event:
-                raise common_ex.BlazarException(
-                    'end_lease event for lease %s not found' % lease_id)
+
+        start_event = db_api.event_get_first_sorted_by_filters(
+            'lease_id',
+            'asc',
+            {
+                'lease_id': lease_id,
+                'event_type': 'start_lease',
+            }
+        )
+        if not start_event:
+            raise common_ex.BlazarException(
+                'start_lease event for lease %s not found' % lease_id)
+
+        end_event = db_api.event_get_first_sorted_by_filters(
+            'lease_id',
+            'asc',
+            {
+                'lease_id': lease_id,
+                'event_type': 'end_lease',
+            }
+        )
+        if not end_event:
+            raise common_ex.BlazarException(
+                'end_lease event for lease %s not found' % lease_id)
+
+        lease_already_started = start_event['status'] != status.event.UNDONE
+        lease_not_started = not lease_already_started
+        lease_already_ended = end_event['status'] != status.event.UNDONE
+        lease_not_ended = not lease_already_ended
+
+        end_lease = lease_already_started and lease_not_ended
+
+        if end_lease:
             db_api.event_update(end_event['id'],
                                 {'status': status.event.IN_PROGRESS})
 
         with trusts.create_ctx_from_trust(lease['trust_id']) as ctx:
-            for reservation in lease['reservations']:
+            reservations = lease['reservations']
+
+            if lease_not_started or lease_not_ended:
+                # Only run the on_end enforcement if we're explicitly
+                # ending the lease for the first time OR if we're terminating
+                # it before the lease ever started. It's important to run
+                # on_end in the second case to inform enforcement that the
+                # lease is no longer in play.
+                allocations = self._existing_allocations(reservations)
+                try:
+                    self.enforcement.on_end(ctx, lease, allocations)
+                except Exception as e:
+                    LOG.error(e)
+
+            for reservation in reservations:
                 if reservation['status'] != status.reservation.DELETED:
                     plugin = self.plugins[reservation['resource_type']]
                     try:
@@ -533,7 +633,14 @@ class ManagerService(service_utils.RPCServer):
         result_in=(status.lease.TERMINATED, status.lease.ERROR))
     def end_lease(self, lease_id, event_id):
         lease = self.get_lease(lease_id)
-        with trusts.create_ctx_from_trust(lease['trust_id']):
+
+        with trusts.create_ctx_from_trust(lease['trust_id']) as ctx:
+            allocations = self._existing_allocations(lease['reservations'])
+            try:
+                self.enforcement.on_end(ctx, lease, allocations)
+            except Exception as e:
+                LOG.error(e)
+
             self._basic_action(lease_id, event_id, 'on_end',
                                status.reservation.DELETED)
 
@@ -579,7 +686,8 @@ class ManagerService(service_utils.RPCServer):
     def _create_reservation(self, values):
         resource_type = values['resource_type']
         if resource_type not in self.plugins:
-            raise exceptions.UnsupportedResourceType(resource_type)
+            raise exceptions.UnsupportedResourceType(
+                resource_type=resource_type)
         reservation_values = {
             'lease_id': values['lease_id'],
             'resource_type': resource_type,
@@ -593,6 +701,59 @@ class ManagerService(service_utils.RPCServer):
         db_api.reservation_update(reservation['id'],
                                   {'resource_id': resource_id})
 
+    def _allocation_candidates(self, lease, reservations):
+        """Returns dict by resource type of reservation candidates."""
+        allocations = {}
+
+        for reservation in reservations:
+            res = reservation.copy()
+            resource_type = reservation['resource_type']
+            res['start_date'] = lease['start_date']
+            res['end_date'] = lease['end_date']
+
+            if resource_type not in self.plugins:
+                raise exceptions.UnsupportedResourceType(
+                    resource_type=resource_type)
+
+            plugin = self.plugins.get(resource_type)
+
+            if not plugin:
+                raise common_ex.BlazarException(
+                    'Invalid plugin names are specified: %s' % resource_type)
+
+            candidate_ids = plugin.allocation_candidates(res)
+
+            allocations[resource_type] = [
+                plugin.get(cid) for cid in candidate_ids]
+
+        return allocations
+
+    def _existing_allocations(self, reservations):
+        allocations = {}
+
+        for reservation in reservations:
+            resource_type = reservation['resource_type']
+
+            if resource_type not in self.plugins:
+                raise exceptions.UnsupportedResourceType(
+                    resource_type=resource_type)
+
+            plugin = self.plugins.get(resource_type)
+
+            if not plugin:
+                raise common_ex.BlazarException(
+                    'Invalid plugin names are specified: %s' % resource_type)
+
+            resource_ids = [
+                x['resource_id'] for x in plugin.list_allocations(
+                    dict(reservation_id=reservation['id']))
+                if x['reservations']]
+
+            allocations[resource_type] = [
+                plugin.get(rid) for rid in resource_ids]
+
+        return allocations
+
     def _send_notification(self, lease, ctx, events=[]):
         payload = notification_api.format_lease_payload(lease)
 
@@ -659,7 +820,8 @@ class ManagerService(service_utils.RPCServer):
             except KeyError:
                 LOG.error("Plugin with resource type %s not found",
                           resource_type)
-                raise exceptions.UnsupportedResourceType(resource_type)
+                raise exceptions.UnsupportedResourceType(
+                    resource_type=resource_type)
         except AttributeError:
             LOG.error("Plugin %s doesn't include method %s",
                       self.plugins[resource_type], method)
