@@ -18,7 +18,6 @@ import datetime
 
 from oslo_context import context
 from oslo_utils import uuidutils
-import six
 
 from blazar.db.sqlalchemy import api as db_api
 from blazar.db.sqlalchemy import utils as db_utils
@@ -27,7 +26,7 @@ from blazar import tests
 
 
 def _get_fake_random_uuid():
-    return six.text_type(uuidutils.generate_uuid())
+    return str(uuidutils.generate_uuid())
 
 
 def _get_fake_lease_uuid():
@@ -202,153 +201,6 @@ class SQLAlchemyDBUtilsTestCase(tests.DBTestCase):
         self.assertEqual('2030-01-01 14:00',
                          reserved_periods[0][1].strftime('%Y-%m-%d %H:%M'))
 
-    def test_availability_time(self):
-        """Find the total availability time."""
-        self._setup_leases()
-        start_date = datetime.datetime.strptime('2030-01-01 09:15',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 10:15',
-                                              '%Y-%m-%d %H:%M')
-        availability_time = db_utils.availability_time('r1',
-                                                       start_date,
-                                                       end_date)
-        self.assertEqual(0 * 60, availability_time.seconds)
-        start_date = datetime.datetime.strptime('2030-01-01 09:15',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 13:45',
-                                              '%Y-%m-%d %H:%M')
-        availability_time = db_utils.availability_time('r1',
-                                                       start_date,
-                                                       end_date)
-        self.assertEqual(150 * 60, availability_time.seconds)
-        start_date = datetime.datetime.strptime('2030-01-01 08:00',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 15:00',
-                                              '%Y-%m-%d %H:%M')
-        availability_time = db_utils.availability_time('r1',
-                                                       start_date,
-                                                       end_date)
-        self.assertEqual(270 * 60, availability_time.seconds)
-
-    def test_reservation_time(self):
-        """Find the total reserved time."""
-        self._setup_leases()
-        start_date = datetime.datetime.strptime('2030-01-01 09:15',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 10:15',
-                                              '%Y-%m-%d %H:%M')
-        reservation_time = db_utils.reservation_time('r1',
-                                                     start_date,
-                                                     end_date)
-        self.assertEqual(60 * 60, reservation_time.seconds)
-        start_date = datetime.datetime.strptime('2030-01-01 09:15',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 13:45',
-                                              '%Y-%m-%d %H:%M')
-        reservation_time = db_utils.reservation_time('r1',
-                                                     start_date,
-                                                     end_date)
-        self.assertEqual(120 * 60, reservation_time.seconds)
-        start_date = datetime.datetime.strptime('2030-01-01 08:00',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 15:00',
-                                              '%Y-%m-%d %H:%M')
-        reservation_time = db_utils.reservation_time('r1',
-                                                     start_date,
-                                                     end_date)
-        self.assertEqual(150 * 60, reservation_time.seconds)
-
-    def test_reservation_ratio(self):
-        """Find the reservation ratio."""
-        self._setup_leases()
-        start_date = datetime.datetime.strptime('2030-01-01 09:15',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 14:30',
-                                              '%Y-%m-%d %H:%M')
-        reservation_time = db_utils.reservation_time('r1',
-                                                     start_date,
-                                                     end_date)
-        availability_time = db_utils.availability_time('r1',
-                                                       start_date,
-                                                       end_date)
-        reservation_ratio = db_utils.reservation_ratio('r1',
-                                                       start_date,
-                                                       end_date)
-        self.assertEqual(reservation_ratio,
-                         float(reservation_time.seconds) /
-                         (end_date - start_date).seconds)
-        self.assertEqual(
-            reservation_ratio,
-            float((end_date - start_date - availability_time).seconds) /
-            (end_date - start_date).seconds)
-
-    def test_number_of_reservations(self):
-        """Find the number of reservations."""
-        self._setup_leases()
-        start_date = datetime.datetime.strptime('2030-01-01 09:15',
-                                                '%Y-%m-%d %H:%M')
-        end_date = datetime.datetime.strptime('2030-01-01 14:30',
-                                              '%Y-%m-%d %H:%M')
-        self.assertEqual(
-            2,
-            db_utils.number_of_reservations('r1', start_date, end_date))
-
-    def test_longest_lease(self):
-        """Find the longest lease."""
-        self._setup_leases()
-        self.assertEqual(
-            db_utils.longest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 08:00'),
-                end_date=_get_datetime('2099-01-01 10:00')),
-            'lease1')
-        self.assertEqual(
-            db_utils.longest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 10:00'),
-                end_date=_get_datetime('2099-01-01 00:00')),
-            'lease3')
-        self.assertEqual(
-            db_utils.longest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 08:00'),
-                end_date=_get_datetime('2030-01-01 11:00')),
-            'lease1')
-        self.assertIsNone(
-            db_utils.longest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 10:15'),
-                end_date=_get_datetime('2030-01-01 13:00')),
-            'lease1')
-
-    def test_shortest_lease(self):
-        """Find the shortest lease."""
-        self._setup_leases()
-        self.assertEqual(
-            db_utils.shortest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 08:00'),
-                end_date=_get_datetime('2099-01-01 10:00')),
-            'lease3')
-        self.assertEqual(
-            db_utils.shortest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 10:00'),
-                end_date=_get_datetime('2099-01-01 00:00')),
-            'lease3')
-        self.assertEqual(
-            db_utils.shortest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 08:00'),
-                end_date=_get_datetime('2030-01-01 11:00')),
-            'lease1')
-        self.assertIsNone(
-            db_utils.shortest_lease(
-                'r1',
-                start_date=_get_datetime('2030-01-01 10:15'),
-                end_date=_get_datetime('2030-01-01 13:00')),
-            'lease1')
-
     def test_get_reservations_by_host_id(self):
         self._setup_leases()
 
@@ -407,73 +259,63 @@ class SQLAlchemyDBUtilsTestCase(tests.DBTestCase):
         self.check_reservation([], ['r4'],
                                '2030-01-01 07:00', '2030-01-01 15:00')
 
-    def test_get_reservation_allocations_by_host_ids(self):
-        def create_allocation_tuple(lease_id):
-            reservation = db_api.reservation_get_all_by_lease_id(lease_id)[0]
-            allocation = db_api.host_allocation_get_all_by_values(
-                reservation_id=reservation['id'])[0]
-            return (reservation['id'], allocation['id'])
+    def _create_allocation_tuple(self, lease_id):
+        reservation = db_api.reservation_get_all_by_lease_id(lease_id)[0]
+        allocation = db_api.host_allocation_get_all_by_values(
+            reservation_id=reservation['id'])[0]
+        return (reservation['id'],
+                reservation['lease_id'],
+                allocation['compute_host_id'])
 
+    def test_get_reservation_allocations_by_host_ids(self):
         self._setup_leases()
 
         # query all allocations of lease1, lease2 and lease3
         expected = [
-            create_allocation_tuple('lease1'),
-            create_allocation_tuple('lease2'),
-            create_allocation_tuple('lease3'),
+            self._create_allocation_tuple('lease1'),
+            self._create_allocation_tuple('lease2'),
+            self._create_allocation_tuple('lease3'),
         ]
         ret = db_utils.get_reservation_allocations_by_host_ids(
             ['r1', 'r2'], '2030-01-01 08:00', '2030-01-01 15:00')
 
-        self.assertListEqual(expected, [(r['id'], a['id']) for r, a in ret])
+        self.assertListEqual(expected, ret)
 
         # query allocations of lease2 and lease3
         expected = [
-            create_allocation_tuple('lease2'),
-            create_allocation_tuple('lease3'),
+            self._create_allocation_tuple('lease2'),
+            self._create_allocation_tuple('lease3'),
         ]
         ret = db_utils.get_reservation_allocations_by_host_ids(
             ['r1', 'r2'], '2030-01-01 11:30', '2030-01-01 15:00')
 
-        self.assertListEqual(expected, [(r['id'], a['id']) for r, a in ret])
+        self.assertListEqual(expected, ret)
 
     def test_get_reservation_allocations_by_host_ids_with_lease_id(self):
-        def create_allocation_tuple(lease_id):
-            reservation = db_api.reservation_get_all_by_lease_id(lease_id)[0]
-            allocation = db_api.host_allocation_get_all_by_values(
-                reservation_id=reservation['id'])[0]
-            return (reservation['id'], allocation['id'])
-
         self._setup_leases()
 
         # query all allocations of lease1, lease2 and lease3
         expected = [
-            create_allocation_tuple('lease1'),
+            self._create_allocation_tuple('lease1'),
         ]
         ret = db_utils.get_reservation_allocations_by_host_ids(
             ['r1', 'r2'], '2030-01-01 08:00', '2030-01-01 15:00', 'lease1')
 
-        self.assertListEqual(expected, [(r['id'], a['id']) for r, a in ret])
+        self.assertListEqual(expected, ret)
 
     def test_get_reservation_allocations_by_host_ids_with_reservation_id(self):
-        def create_allocation_tuple(lease_id):
-            reservation = db_api.reservation_get_all_by_lease_id(lease_id)[0]
-            allocation = db_api.host_allocation_get_all_by_values(
-                reservation_id=reservation['id'])[0]
-            return (reservation['id'], allocation['id'])
-
         self._setup_leases()
         reservation1 = db_api.reservation_get_all_by_lease_id('lease1')[0]
 
         # query allocations of lease1
         expected = [
-            create_allocation_tuple('lease1'),
+            self._create_allocation_tuple('lease1'),
         ]
         ret = db_utils.get_reservation_allocations_by_host_ids(
             ['r1', 'r2'], '2030-01-01 08:00', '2030-01-01 15:00',
             reservation_id=reservation1['id'])
 
-        self.assertListEqual(expected, [(r['id'], a['id']) for r, a in ret])
+        self.assertListEqual(expected, ret)
 
     def test_get_plugin_reservation_with_host(self):
         patch_host_reservation_get = self.patch(db_api, 'host_reservation_get')
@@ -497,6 +339,3 @@ class SQLAlchemyDBUtilsTestCase(tests.DBTestCase):
     def test_get_plugin_reservation_with_invalid(self):
         self.assertRaises(mgr_exceptions.UnsupportedResourceType,
                           db_utils.get_plugin_reservation, 'invalid', 'id1')
-
-# TODO(frossigneux) longest_availability
-# TODO(frossigneux) shortest_availability
